import {useEffect, useRef, useState} from "react";
import "./styles.sass";

export const MusicSlider = ({ isReadOnly = false, onFrequencyChange, frequency: externalFrequency }) => {
    const [frequency, setFrequency] = useState(externalFrequency || 440);
    const [isPlaying, setIsPlaying] = useState(false);
    const [dragging, setDragging] = useState(false);
    const audioContextRef = useRef(null);
    const oscillatorRef = useRef(null);
    const gainNodeRef = useRef(null);
    const sliderRef = useRef(null);
    const hasInteractedRef = useRef(false);

    useEffect(() => {
        const initAudioContext = () => {
            if (!audioContextRef.current && !hasInteractedRef.current) {
                hasInteractedRef.current = true;
                audioContextRef.current = new (window.AudioContext || window.webkitAudioContext)();
                if (!oscillatorRef.current) {
                    startAudio();
                }
                document.removeEventListener('click', initAudioContext);
                document.removeEventListener('touchstart', initAudioContext);
                document.removeEventListener('keydown', initAudioContext);
            }
        };

        document.addEventListener('click', initAudioContext);
        document.addEventListener('touchstart', initAudioContext);
        document.addEventListener('keydown', initAudioContext);

        return () => {
            document.removeEventListener('click', initAudioContext);
            document.removeEventListener('touchstart', initAudioContext);
            document.removeEventListener('keydown', initAudioContext);
        };
    }, []);

    useEffect(() => {
        if (externalFrequency !== undefined && !dragging) {
            setFrequency(externalFrequency);

            if (audioContextRef.current) {
                if (!isPlaying) {
                    startAudio();
                } else if (oscillatorRef.current) {
                    oscillatorRef.current.frequency.setValueAtTime(
                        externalFrequency, 
                        audioContextRef.current.currentTime
                    );
                }
            }
        }
    }, [externalFrequency, dragging, isPlaying]);

    const handleMouseDown = (e) => {
        if (isReadOnly) return;
        
        setDragging(true);
        handleFrequencyChange(e);

        if (!audioContextRef.current) {
            audioContextRef.current = new (window.AudioContext || window.webkitAudioContext)();
        }

        if (!isPlaying) {
            startAudio();
        }
    };

    const handleMouseUp = () => {
        setDragging(false);
    };

    const handleFrequencyChange = (e) => {
        if (isReadOnly) return;
        
        const rect = sliderRef.current.getBoundingClientRect();
        const x = e.clientX - rect.left;
        const clampedX = Math.max(0, Math.min(x, rect.width));
        const newFrequency = 20 + (clampedX / rect.width) * 1980;
        setFrequency(newFrequency);

        if (onFrequencyChange) {
            onFrequencyChange(newFrequency);
        }
    };

    useEffect(() => {
        const handleMouseMove = (e) => dragging && handleFrequencyChange(e);
        const handleMouseUpGlobal = () => dragging && handleMouseUp();

        document.addEventListener('mousemove', handleMouseMove);
        document.addEventListener('mouseup', handleMouseUpGlobal);

        return () => {
            document.removeEventListener('mousemove', handleMouseMove);
            document.removeEventListener('mouseup', handleMouseUpGlobal);
        };
    }, [dragging]);

    useEffect(() => {
        if (isPlaying && oscillatorRef.current && audioContextRef.current) {
            oscillatorRef.current.frequency.setValueAtTime(
                frequency, 
                audioContextRef.current.currentTime
            );
        }
    }, [frequency, isPlaying]);

    const startAudio = () => {
        if (!audioContextRef.current) {
            try {
                audioContextRef.current = new (window.AudioContext || window.webkitAudioContext)();
            } catch (e) {
                console.error("AudioContext could not be created:", e);
                return;
            }
        }

        if (audioContextRef.current.state === 'suspended') {
            audioContextRef.current.resume().catch(err => {
                console.error("Could not resume AudioContext:", err);
            });
        }
        
        try {
            if (!oscillatorRef.current) {
                oscillatorRef.current = audioContextRef.current.createOscillator();
                oscillatorRef.current.type = 'sine';
                oscillatorRef.current.frequency.setValueAtTime(
                    frequency, 
                    audioContextRef.current.currentTime
                );

                gainNodeRef.current = audioContextRef.current.createGain();
                gainNodeRef.current.gain.setValueAtTime(0.5, audioContextRef.current.currentTime);

                oscillatorRef.current.connect(gainNodeRef.current);
                gainNodeRef.current.connect(audioContextRef.current.destination);
                oscillatorRef.current.start();
                console.log("Audio started successfully");
            } else {
                oscillatorRef.current.frequency.setValueAtTime(
                    frequency, 
                    audioContextRef.current.currentTime
                );
            }
            setIsPlaying(true);
        } catch (e) {
            console.error("Error starting audio:", e);
        }
    };

    const stopAudio = () => {
        if (oscillatorRef.current) {
            try {
                oscillatorRef.current.stop();
                oscillatorRef.current.disconnect();
                oscillatorRef.current = null;
                setIsPlaying(false);
                console.log("Audio stopped");
            } catch (e) {
                console.error("Error stopping audio:", e);
            }
        }
    };

    useEffect(() => {
        return () => {
            if (oscillatorRef.current) {
                try {
                    oscillatorRef.current.stop();
                    oscillatorRef.current.disconnect();
                } catch (e) {
                    console.error("Error cleaning up oscillator:", e);
                }
            }
            
            if (audioContextRef.current && audioContextRef.current.state !== 'closed') {
                try {
                    audioContextRef.current.close();
                } catch (e) {
                    console.error("Error closing AudioContext:", e);
                }
            }
        };
    }, []);

    useEffect(() => {
        return () => {
            if (isPlaying) {
                stopAudio();
            }
        };
    }, [isPlaying]);

    return (
        <div className={`otamatone-container ${isReadOnly ? 'read-only' : ''}`}>
            <div 
                className="otamatone" 
                onMouseDown={handleMouseDown}
                style={{ cursor: isReadOnly ? 'default' : 'pointer' }}
            >
                <div className="otamatone-face">
                    <div className="otamatone-mouth" style={{
                        height: `${10 + (frequency / 2000) * 40}px`,
                        width: `${10 + (frequency / 2000) * 40}px`
                    }}></div>
                    <div className="otamatone-eyes">
                        <div className="eye left-eye"></div>
                        <div className="eye right-eye"></div>
                    </div>
                </div>
                <div className="otamatone-neck" ref={sliderRef}>
                    <div className="frequency-indicator" style={{left: `${(frequency - 20) / 1980 * 100}%`}}></div>
                    <div className="note-marker" style={{left: '10%', pointerEvents: 'none'}}>♩</div>
                    <div className="note-marker" style={{left: '50%', pointerEvents: 'none'}}>♪</div>
                    <div className="note-marker" style={{left: '90%', pointerEvents: 'none'}}>♫</div>
                </div>
            </div>
        </div>
    );
};