import "./styles.sass";
import ActionCard from "@/pages/Home/components/ActionCard";
import JoinForm from "@/pages/Home/components/JoinForm";
import CreateForm from "@/pages/Home/components/CreateForm";
import {faArrowRightToBracket, faPlusSquare} from "@fortawesome/free-solid-svg-icons";
import {StateContext} from "@/common/contexts/StateContext";
import {SocketContext} from "@/common/contexts/SocketContext";
import {useContext, useState, useEffect} from "react";

export const Home = () => {
    const {setCurrentState} = useContext(StateContext);
    const {connect, send, on, connected} = useContext(SocketContext);
    const [homeState, setHomeState] = useState("initial"); // initial, joining, creating
    const [error, setError] = useState("");

    useEffect(() => {
        if (!connected) {
            connect();
        }
    }, [connected, connect]);

    useEffect(() => {
        connect();
        
        const handleRoomCreated = (roomId) => {
            console.log("Room created", roomId);
            setCurrentState("WaitingRoom");
        };
        
        const handleRoomJoined = (roomId) => {
            console.log("Room joined", roomId);
            setCurrentState("WaitingRoom");
        };
        
        const handleRoomNotFound = (roomId) => {
            setError(`Room ${roomId} not found`);
        };
        
        const handleRoomClosed = (roomId) => {
            setError(`Room ${roomId} is already in game`);
        };
        
        const cleanupRoomCreated = on("room-created", handleRoomCreated);
        const cleanupRoomJoined = on("room-joined", handleRoomJoined);
        const cleanupRoomNotFound = on("room-not-found", handleRoomNotFound);
        const cleanupRoomClosed = on("room-closed", handleRoomClosed);
        
        return () => {
            cleanupRoomCreated();
            cleanupRoomJoined();
            cleanupRoomNotFound();
            cleanupRoomClosed();
        };
    }, [connect, setCurrentState, on]);

    const handleJoinClick = () => {
        setHomeState("joining");
        setError("");
    };

    const handleCreateClick = () => {
        setHomeState("creating");
        setError("");
    };

    const handleBack = () => {
        setHomeState("initial");
        setError("");
    };

    const handleJoinSubmit = (name, roomCode) => {
        send("join-room", {roomId: roomCode.toUpperCase(), name});
    };

    const handleCreateSubmit = (name) => {
        send("create-room", {name});
    };

    return (
        <div className="home-page">
            <div className="background-overlay">
                <div className="rotating-gradient"></div>
            </div>
            
            <div className={`logo-container ${homeState !== 'initial' ? 'shifted' : ''}`}>
                <h1 className="logo">ToneGuessr</h1>
                <p className="tagline">Das Frequenzspiel für Marco :3</p>
            </div>
            
            <div className={`content-container ${homeState !== 'initial' ? 'active' : ''}`}>
                <div className={`action-area ${homeState !== 'initial' ? 'hidden' : ''}`}>
                    <ActionCard title="Beitreten" icon={faArrowRightToBracket} onClick={handleJoinClick} />
                    <ActionCard title="Erstellen" icon={faPlusSquare} onClick={handleCreateClick} />
                </div>
                
                <div className={`form-container ${homeState === 'joining' ? 'active' : ''}`}>
                    {homeState === 'joining' && (
                        <JoinForm onSubmit={handleJoinSubmit} onBack={handleBack} error={error} />
                    )}
                </div>
                
                <div className={`form-container ${homeState === 'creating' ? 'active' : ''}`}>
                    {homeState === 'creating' && (
                        <CreateForm onSubmit={handleCreateSubmit} onBack={handleBack} error={error} />
                    )}
                </div>
            </div>
        </div>
    );
}
